@@ -12,11 +12,11 @@ Web::ACL - A helper for creating basic apikey/slug/IP based ACLs.
 
 =head1 VERSION
 
-Version 0.0.2
+Version 0.1.0
 
 =cut
 
-our $VERSION = '0.0.2';
+our $VERSION = '0.1.0';
 
 =head1 SYNOPSIS
 
@@ -48,7 +48,7 @@ our $VERSION = '0.0.2';
             derpderp=>{
 				ip_auth       => 0,
 				slug_auth     => 1,
-				require_ip    => 1,
+				require_ip    => 0,
 				require_slug  => 0,
 				final         => 1,
 				slugs         => ['derp'],
@@ -124,7 +124,9 @@ Initiates the object.
         - Default :: {
 			'undef' => {
 				ip_auth       => 0,
+				path_auth     => 0,
 				slug_auth     => 0,
+				ua_auth       => 0,
 				require_ip    => 0,
 				require_slug  => 0,
 				final         => 0,
@@ -132,10 +134,16 @@ Initiates the object.
 				slugs_regex   => [],
 				allow_subnets => [],
 				deny_subnets  => [],
+				ua_regex_allow    => [],
+				ua_regex_deny     => [],
+				paths_regex_allow => [],
+				paths_regex_deny  => [],
 			},
 			'nonexistent' => {
 				ip_auth       => 0,
+				path_auth      => 0,
 				slug_auth     => 0,
+				ua_auth       => 0,
 				require_ip    => 0,
 				require_slug  => 0,
 				final         => 0,
@@ -143,6 +151,10 @@ Initiates the object.
 				slugs_regex   => [],
 				allow_subnets => [],
 				deny_subnets  => [],
+				ua_regex_allow    => [],
+				ua_regex_deny     => [],
+				paths_regex_allow => [],
+				paths_regex_deny  => [],
 			},
 		}
 
@@ -163,32 +175,49 @@ sub new {
 				1 => 'ACLnotHash',
 				2 => 'ACLitemNotArray',
 				3 => 'subnetError',
+				4 => 'ACLnotString',
 			},
 			fatal_flags      => {},
 			perror_not_fatal => 0,
 		},
 		acl => {
 			'undef' => {
-				ip_auth       => 0,
-				slug_auth     => 0,
-				require_ip    => 0,
-				require_slug  => 0,
-				final         => 0,
-				slugs         => [],
-				slugs_regex   => [],
-				allow_subnets => [],
-				deny_subnets  => [],
+				ip_auth           => 0,
+				slug_auth         => 0,
+				ua_auth           => 0,
+				path_auth         => 0,
+				require_ip        => 0,
+				require_slug      => 0,
+				require_ua        => 0,
+				require_path      => 0,
+				final             => 0,
+				slugs             => [],
+				slugs_regex       => [],
+				allow_subnets     => [],
+				deny_subnets      => [],
+				ua_regex_allow    => [],
+				ua_regex_deny     => [],
+				paths_regex_allow => [],
+				paths_regex_deny  => [],
 			},
 			'nonexistent' => {
-				ip_auth       => 0,
-				slug_auth     => 0,
-				require_ip    => 0,
-				require_slug  => 0,
-				final         => 0,
-				slugs         => [],
-				slugs_regex   => [],
-				allow_subnets => [],
-				deny_subnets  => [],
+				ip_auth           => 0,
+				slug_auth         => 0,
+				ua_auth           => 0,
+				path_auth         => 0,
+				require_ip        => 0,
+				require_slug      => 0,
+				require_ua        => 0,
+				require_path      => 0,
+				final             => 0,
+				slugs             => [],
+				slugs_regex       => [],
+				allow_subnets     => [],
+				deny_subnets      => [],
+				ua_regex_allow    => [],
+				ua_regex_deny     => [],
+				paths_regex_allow => [],
+				paths_regex_deny  => [],
 			},
 		},
 	};
@@ -198,21 +227,28 @@ sub new {
 
 		my @acl_keys = keys( %{ $opts{acl} } );
 		foreach my $acl (@acl_keys) {
-			if ( !defined( $opts{acl}{$acl}{ip_auth} ) ) {
-				$opts{acl}{$acl}{ip_auth} = 0;
-			}
-
-			if ( !defined( $opts{acl}{$acl}{slug_auth} ) ) {
-				$opts{acl}{$acl}{slug_auth} = 0;
-			}
-
-			if ( !defined( $opts{acl}{$acl}{require_ip} ) ) {
-				$opts{acl}{$acl}{require_ip} = 0;
-			}
-
-			if ( !defined( $opts{acl}{$acl}{require_slug} ) ) {
-				$opts{acl}{$acl}{require_slug} = 0;
-			}
+			# check boolean items and define if undef
+			my @keys_that_are_boolean = (
+				'ip_auth', 'require_ip', 'slug_auth', 'require_slug', 'path_auth', 'require_path',
+				'ua_auth', 'require_ua', 'final',
+			);
+			foreach my $boolean_key (@keys_that_are_boolean) {
+				if ( !defined( $opts{acl}{$acl}{$boolean_key} ) ) {
+					$opts{acl}{$acl}{$boolean_key} = 0;
+				} else {
+					if ( ref( $opts{acl}{$acl}{$boolean_key} ) ne '' ) {
+						$self->{perror} = 1;
+						$self->{error}  = 4;
+						$self->{errorString}
+							= '$opts{acl}{acl}{'
+							. $boolean_key
+							. '} is not ref "", but "'
+							. ref( $opts{acl}{$acl}{$boolean_key} ) . '"';
+						$self->warn;
+						return;
+					} ## end if ( ref( $opts{acl}{$acl}{$boolean_key} )...)
+				} ## end else [ if ( !defined( $opts{acl}{$acl}{$boolean_key...}))]
+			} ## end foreach my $boolean_key (@keys_that_are_boolean)
 
 			if ( !defined( $opts{acl}{$acl}{final} ) && ( $acl eq 'undef' || $acl eq 'nonexistent' ) ) {
 				$opts{acl}{$acl}{final} = 0;
@@ -220,87 +256,70 @@ sub new {
 				$opts{acl}{$acl}{final} = 1;
 			}
 
-			if ( !defined( $opts{acl}{$acl}{slugs} ) ) {
-				$opts{acl}{$acl}{slugs} = [];
-			} elsif ( ref( $opts{acl}{$acl}{slugs} ) ne 'ARRAY' ) {
-				$self->{perror} = 1;
-				$self->{error}  = 2;
-				$self->{errorString}
-					= '$opts{acl}{acl}{slugs} is not ref ARRAY, but "' . ref( $opts{acl}{$acl}{slugs} ) . '"';
-				$self->warn;
-				return;
-			}
-
-			if ( !defined( $opts{acl}{$acl}{slugs_regex} ) ) {
-				$opts{acl}{$acl}{slugs_regex} = [];
-			} elsif ( ref( $opts{acl}{$acl}{slugs_regex} ) ne 'ARRAY' ) {
-				$self->{perror}      = 1;
-				$self->{error}       = 2;
-				$self->{errorString} = '$opts{acl}{acl}{slugs_regex} is not ref ARRAY, but "'
-					. ref( $opts{acl}{$acl}{slugs_regex} ) . '"';
-				$self->warn;
-				return;
-			}
-
-			if ( !defined( $opts{acl}{$acl}{slugs_regex} ) ) {
-				$opts{acl}{$acl}{deny_subnets} = [];
-			} elsif ( ref( $opts{acl}{$acl}{slugs_regex} ) ne 'ARRAY' ) {
-				$self->{perror}      = 1;
-				$self->{error}       = 2;
-				$self->{errorString} = '$opts{acl}{acl}{slugs_regex} is not ref ARRAY, but "'
-					. ref( $opts{acl}{$acl}{slugs_regex} ) . '"';
-				$self->warn;
-				return;
-			}
-
-			if ( !defined( $opts{acl}{$acl}{allow_subnets} ) ) {
-				$opts{acl}{$acl}{allow_subnets} = [];
-			} elsif ( ref( $opts{acl}{$acl}{allow_subnets} ) ne 'ARRAY' ) {
-				$self->{perror}      = 1;
-				$self->{error}       = 2;
-				$self->{errorString} = '$opts{acl}{acl}{allow_subnets} is not ref ARRAY, but "'
-					. ref( $opts{acl}{$acl}{allow_subnets} ) . '"';
-				$self->warn;
-				return;
-			}
-
-			if ( !defined( $opts{acl}{$acl}{deny_subnets} ) ) {
-				$opts{acl}{$acl}{deny_subnets} = [];
-			} elsif ( ref( $opts{acl}{$acl}{deny_subnets} ) ne 'ARRAY' ) {
-				$self->{perror}      = 1;
-				$self->{error}       = 2;
-				$self->{errorString} = '$opts{acl}{acl}{deny_subnets} is not ref ARRAY, but "'
-					. ref( $opts{acl}{$acl}{deny_subnets} ) . '"';
-				$self->warn;
-				return;
-			}
+			# check array items and error if they are not a array
+			# if undef, create a empty array
+			my @keys_that_are_arrays = (
+				'slugs',            'ua_regex_allow', 'ua_regex_deny', 'paths_regex_allow',
+				'paths_regex_deny', 'slugs_regex',    'allow_subnets', 'deny_subnets',
+			);
+			foreach my $array_key (@keys_that_are_arrays) {
+				if ( !defined( $opts{acl}{$acl}{$array_key} ) ) {
+					$opts{acl}{$acl}{$array_key} = [];
+				} elsif ( ref( $opts{acl}{$acl}{$array_key} ) ne 'ARRAY' ) {
+					$self->{perror} = 1;
+					$self->{error}  = 2;
+					$self->{errorString}
+						= '$opts{acl}{acl}{'
+						. $array_key
+						. '} is not ref ARRAY, but "'
+						. ref( $opts{acl}{$acl}{$array_key} ) . '"';
+					$self->warn;
+					return;
+				} ## end elsif ( ref( $opts{acl}{$acl}{$array_key} ) ne...)
+			} ## end foreach my $array_key (@keys_that_are_arrays)
 		} ## end foreach my $acl (@acl_keys)
 
 		if ( !defined( $opts{acl}{'undef'} ) ) {
 			$opts{acl}{'undef'} = {
-				ip_auth       => 0,
-				slug_auth     => 0,
-				require_ip    => 0,
-				require_slug  => 0,
-				final         => 0,
-				slugs         => [],
-				slugs_regex   => [],
-				allow_subnets => [],
-				deny_subnets  => [],
+				ip_auth           => 0,
+				slug_auth         => 0,
+				ua_auth           => 0,
+				path_auth         => 0,
+				require_ip        => 0,
+				require_slug      => 0,
+				require_ua        => 0,
+				require_path      => 0,
+				final             => 0,
+				slugs             => [],
+				slugs_regex       => [],
+				allow_subnets     => [],
+				deny_subnets      => [],
+				ua_regex_allow    => [],
+				ua_regex_deny     => [],
+				paths_regex_allow => [],
+				paths_regex_deny  => [],
 			};
 		} ## end if ( !defined( $opts{acl}{'undef'} ) )
 
 		if ( !defined( $opts{acl}{'nonexistent'} ) ) {
 			$opts{acl}{'nonexistent'} = {
-				ip_auth       => 0,
-				slug_auth     => 0,
-				require_ip    => 0,
-				require_slug  => 0,
-				final         => 0,
-				slugs         => [],
-				slugs_regex   => [],
-				allow_subnets => [],
-				deny_subnets  => [],
+				ip_auth           => 0,
+				slug_auth         => 0,
+				ua_auth           => 0,
+				path_auth         => 0,
+				require_ip        => 0,
+				require_slug      => 0,
+				require_ua        => 0,
+				require_path      => 0,
+				final             => 0,
+				slugs             => [],
+				slugs_regex       => [],
+				allow_subnets     => [],
+				deny_subnets      => [],
+				ua_regex_allow    => [],
+				ua_regex_deny     => [],
+				paths_regex_allow => [],
+				paths_regex_deny  => [],
 			};
 		} ## end if ( !defined( $opts{acl}{'nonexistent'} ))
 
@@ -334,14 +353,18 @@ sub new {
 sub check {
 	my ( $self, %opts ) = @_;
 
+	# set what API key we should check against
+	# undef and nonexistent are two special ones set based on apikey being undef or not being a defined one
 	if ( !defined( $opts{apikey} ) ) {
 		$opts{apikey} = 'undef';
 	} elsif ( !defined( $self->{acl}{ $opts{apikey} } ) ) {
 		$opts{apikey} = 'nonexistent';
 	}
 
+	# get the slugs
 	my @slugs;
 	if ( !defined( $opts{slugs} ) ) {
+		# no point in continuing if we require a slug and
 		if ( $self->{acl}{ $opts{apikey} }{require_slug} ) {
 			return 0;
 		}
@@ -359,6 +382,27 @@ sub check {
 		}
 	} ## end else [ if ( !defined( $opts{slugs} ) ) ]
 
+	# ensure we have a UA if required
+	my $ua;
+	if ( !defined( $opts{ua} ) ) {
+		if ( $self->{acl}{ $opts{apikey} }{require_ua} ) {
+			return 0;
+		}
+	} else {
+		$ua = $opts{ua};
+	}
+
+	# ensure we have a path if required
+	my $path;
+	if ( !defined( $opts{path} ) ) {
+		if ( $self->{acl}{ $opts{apikey} }{require_path} ) {
+			return 0;
+		}
+	} else {
+		$path = $opts{path};
+	}
+
+	# ensure we have a IP if required
 	my $ip;
 	if ( !defined( $opts{ip} ) ) {
 		if ( $self->{acl}{ $opts{apikey} }{require_ip} ) {
@@ -368,6 +412,7 @@ sub check {
 		$ip = $opts{ip};
 	}
 
+	# process IP
 	if ( defined($ip) && $self->{acl}{ $opts{apikey} }{ip_auth} ) {
 		my $denied_subnets;
 		eval { $denied_subnets = subnet_matcher( @{ $self->{acl}{ $opts{apikey} }{deny_subnets} } ); };
@@ -398,6 +443,7 @@ sub check {
 		return 0;
 	}
 
+	# process slugs
 	if ( $self->{acl}{ $opts{apikey} }{slug_auth} ) {
 		my %matched_slugs;
 
@@ -427,6 +473,78 @@ sub check {
 		}
 	} ## end if ( $self->{acl}{ $opts{apikey} }{slug_auth...})
 
+	# process useragent info
+	if ( defined($ua) && $self->{acl}{ $opts{apikey} }{ua_auth} ) {
+		# process allows if we have any
+		if ( defined( $self->{acl}{ $opts{apikey} }{ua_regex_allow}[0] ) ) {
+			my $ua_matched = 0;
+			foreach my $item ( @{ $self->{acl}{ $opts{apikey} }{ua_regex_allow} } ) {
+				eval {
+					if ( $ua =~ /$item/ ) {
+						$ua_matched = 1;
+					}
+				};
+			}
+			# if no allowed regexp matched, deny it
+			if ( !$ua_matched ) {
+				return 0;
+			}
+		} ## end if ( defined( $self->{acl}{ $opts{apikey} ...}))
+		# process allows if we have any
+		if ( defined( $self->{acl}{ $opts{apikey} }{ua_regex_deny}[0] ) ) {
+			my $ua_matched = 0;
+			foreach my $item ( @{ $self->{acl}{ $opts{apikey} }{ua_regex_dney} } ) {
+				eval {
+					if ( $ua =~ /$item/ ) {
+						$ua_matched = 1;
+					}
+				};
+			}
+			# if any deny regexp matched, deny it
+			if ($ua_matched) {
+				return 0;
+			}
+		} ## end if ( defined( $self->{acl}{ $opts{apikey} ...}))
+	} elsif ( !defined($ua) && $self->{acl}{ $opts{apikey} }{ua_auth} ) {
+		return 0;
+	}
+
+	# process path info
+	if ( defined($path) && $self->{acl}{ $opts{apikey} }{path_auth} ) {
+		# process allows if we have any
+		if ( defined( $self->{acl}{ $opts{apikey} }{path_regex_allow}[0] ) ) {
+			my $path_matched = 0;
+			foreach my $item ( @{ $self->{acl}{ $opts{apikey} }{path_regex_allow} } ) {
+				eval {
+					if ( $path =~ /$item/ ) {
+						$path_matched = 1;
+					}
+				};
+			}
+			# if no allowed regexp matched, deny it
+			if ( !$path_matched ) {
+				return 0;
+			}
+		} ## end if ( defined( $self->{acl}{ $opts{apikey} ...}))
+		# process allows if we have any
+		if ( defined( $self->{acl}{ $opts{apikey} }{ua_regex_deny}[0] ) ) {
+			my $path_matched = 0;
+			foreach my $item ( @{ $self->{acl}{ $opts{apikey} }{path_regex_dney} } ) {
+				eval {
+					if ( $path =~ /$item/ ) {
+						$path_matched = 1;
+					}
+				};
+			}
+			# if any deny regexp matched, deny it
+			if ($path_matched) {
+				return 0;
+			}
+		} ## end if ( defined( $self->{acl}{ $opts{apikey} ...}))
+	} elsif ( !defined($path) && $self->{acl}{ $opts{apikey} }{path_auth} ) {
+		return 0;
+	}
+
 	return $self->{acl}{ $opts{apikey} }{final};
 } ## end sub check
 
@@ -440,15 +558,27 @@ Slugs should be though of a freeform text field for access check. Function name
     - ip_auth :: Use IP for authing. If false, the IP will not be checked.
         - Default :: 0
 
+    - path_auth :; Use the path for authing. If false it won't be checked.
+        - Default :: 0
+
     - slug_auth :; Use the slug for authing. If false it won't be checked.
         - Default :: 0
 
+    - ua_auth :; Use the UA for authing. If false it won't be checked.
+        - Default :: 0
+
     - require_ip :: Require a value for IP to be specified.
         - Default :: 0
 
+    - require_path :: Require a value for path to be specified.
+        - Default :: 0
+
     - require_slug :: Require a value for slug to be specified.
         - Default :: 0
 
+    - require_slug :: Require a value for UA to be specified.
+        - Default :: 0
+
     - final :: The return value for if none of the auth checks are denied.
          - Default for 'undef'/'nonexistent' apikeys:: 0
          - Default for other apikeys:: 1
@@ -465,6 +595,18 @@ Slugs should be though of a freeform text field for access check. Function name
     - deny_subnets :: Denied subnets for remote IPs. This is a array of CIDRs.
         - Default :: []
 
+    - paths_regex_allow :: Allowed paths.
+        - Default :: []
+
+    - paths_regex_deny :: Denied paths.
+        - Default :: []
+
+    - ua_regex_allow :: Allowed UAs.
+        - Default :: []
+
+    - ua_regex_deny :: Denied UAs.
+        - Default :: []
+
 There are two special ones for the ACL hash. Those are 'undef' and 'nonexistent'
 and they should not be used as API keys. These are for in the instances that
 the apikey for the checkis undef or if specified and does not exist 'nonexistent'
@@ -511,6 +653,10 @@ The expected item is expected to be of the ref type ARRAY.
 
 Could not init sub_matcher.
 
+=head2 4, ACLnotString
+
+'acl' as passed to new is not of the of the ref type ''.
+
 =head1 AUTHOR
 
 Zane C. Bowers-Hadley, C<< <vvelox at vvelox.net> >>
